
; FASM SOURCE FILE
; http://flatassembler.net

; FPU conditional jump macros
; version 1.03

; written by @L.CHEMIST (Andrey A. Meshkov)
; maalchemist@gmail.com
; maa-mail@yandex.ru

; ------------------------------------------------------------------------------

; NOTE: jump macros modify EAX register

; ------------------------------------------------------------------------------

; f[n]stsw ax
; ah                        al
; 07 06 05 04 03 02 01 00   07 06 05 04 03 02 01 00
; ?? C3 ?? ?? ?? C2 C1 C0   ?? ?? PE UE OE ZE DE IE
; sahf
; SF ZF ?? AF ?? PF ?? CF
;
; ------------------------------------------------------------------------------
;
; fcom[p] {ftst}
; C3 C2 C1 C0  meaning                      f[n]stsw ax      test $01  $40  $41
; 0  0  ?  0   ST > source {0}              ? 0 ? ? ? 0 ? 0        z    z    z
; 0  0  ?  1   ST < source {0}              ? 0 ? ? ? 0 ? 1        nz   z    nz
; 1  0  ?  0   ST = source {0}              ? 1 ? ? ? 0 ? 0        z    nz   nz
;
; 1  1  ?  1   ST not comparable to source  ? 1 ? ? ? 1 ? 1        nz   nz   nz
;
;                                      $01  0 0 0 0 0 0 0 1  $01
;                                      $40  0 1 0 0 0 0 0 0  $40
;                                      $41  0 1 0 0 0 0 0 1  $41
;
;                                    FLAGS  S Z ? A ? P ? C  FLAGS
;
; ------------------------------------------------------------------------------
;
;               fld     a               fld     a
;               fcom    source          fcom    source
;               fnstsw  ax              fnstsw  ax
;
; ST  > source  test    ah, $41         sahf    CF=0 & ZF=0
;               jz      .......         ja      .......
;
; ST >= source  test    ah, $01         sahf    CF=0
;               jz      .......         jae     .......
;
; ST  < source  test    ah, $01         sahf    CF=1
;               jnz     .......         jb      .......
;
; ST <= source  test    ah, $41         sahf    CF=1 | ZF=1
;               jnz     .......         jbe     .......
;
; ST == source  test    ah, $40         sahf    ZF=1
;               jnz     .......         je      .......
;
; ST <> source  test    ah, $40         sahf    ZF=0
;               jz      .......         jne     .......

; ------------------------------------------------------------------------------

; Jump if ST > source

macro   fjg     a
{
        fstsw   ax
        fajg    a
}
macro   fnjg    a
{
        fnstsw  ax
        fajg    a
}
macro   fajg    a
{
        sahf
        ja      a
;       test    ah, $41                         ; 0100 0001
;       jz      a
}

fjnle   fix     fjg                             ; Jump if Not Less or Equal
fnjnle  fix     fnjg                            ; Jump if Not Less or Equal
fajnle  fix     fajg                            ; Jump if Not Less or Equal

fja     fix     fjg                             ; Jump if Above
fnja    fix     fnjg                            ; Jump if Above
faja    fix     fajg                            ; Jump if Above

fjnbe   fix     fjg                             ; Jump if Not Below or Equal
fnjnbe  fix     fnjg                            ; Jump if Not Below or Equal
fajnbe  fix     fajg                            ; Jump if Not Below or Equal

; ------------------------------------------------------------------------------

; Jump if ST >= source

macro   fjge    a
{
        fstsw   ax
        fajge   a
}
macro   fnjge   a
{
        fnstsw  ax
        fajge   a
}
macro   fajge   a
{
        sahf
        jae     a
;       test    ah, $01                         ; 0000 0001
;       jz      a
}

fjnl    fix     fjge                            ; Jump if Not Less
fnjnl   fix     fnjge                           ; Jump if Not Less
fajnl   fix     fajge                           ; Jump if Not Less

fjae    fix     fjge                            ; Jump if Above or Equal
fnjae   fix     fnjge                           ; Jump if Not Less
fajae   fix     fajge                           ; Jump if Not Less

fjnb    fix     fjge                            ; Jump if Not Below
fnjnb   fix     fnjge                           ; Jump if Not Less
fajnb   fix     fajge                           ; Jump if Not Less

; ------------------------------------------------------------------------------

; Jump if ST < source

macro   fjl     a
{
        fstsw   ax
        fajl    a
}
macro   fnjl    a
{
        fnstsw  ax
        fajl    a
}
macro   fajl    a
{
        sahf
        jb      a
;       test    ah, $01                         ; 0000 0001
;       jnz     a
}

fjnge   fix     fjl                             ; Jump if Not Greater or Equal
fnjnge  fix     fnjl                            ; Jump if Not Greater or Equal
fajnge  fix     fajl                            ; Jump if Not Greater or Equal

fjb     fix     fjl                             ; Jump if Below
fnjb    fix     fnjl                            ; Jump if Below
fajb    fix     fajl                            ; Jump if Below

fjnae   fix     fjl                             ; Jump if Not Above or Equal
fnjnae  fix     fnjl                            ; Jump if Not Above or Equal
fajnae  fix     fajl                            ; Jump if Not Above or Equal

; ------------------------------------------------------------------------------

; Jump if ST <= source

macro   fjle    a
{
        fstsw   ax
        fajle   a
}
macro   fnjle   a
{
        fnstsw  ax
        fajle   a
}
macro   fajle   a
{
        sahf
        jbe     a
;       test    ah, $41                         ; 0100 0001
;       jnz     a
}

fjng    fix     fjle                            ; Jump if Not Greater
fnjng   fix     fnjle                           ; Jump if Not Greater
fajng   fix     fajle                           ; Jump if Not Greater

fjbe    fix     fjle                            ; Jump if Below or Equal
fnjbe   fix     fnjle                           ; Jump if Below or Equal
fajbe   fix     fajle                           ; Jump if Below or Equal

fnjna   fix     fjle                            ; Jump if Not Above
fnjna   fix     fnjle                           ; Jump if Not Above
fajna   fix     fajle                           ; Jump if Not Above

; ------------------------------------------------------------------------------

; Jump if ST = source

macro   fje     a
{
        fstsw   ax
        faje    a
}
macro   fnje    a
{
        fnstsw  ax
        faje    a
}
macro   faje    a
{
        sahf
        je      a
;       test    ah, $40                         ; 0100 0000
;       jnz     a
}

fjz     fix     fje                             ; Jump if Zero
fnjz    fix     fnje                            ; Jump if Zero
fajz    fix     faje                            ; Jump if Zero

; ------------------------------------------------------------------------------

; Jump if ST <> source

macro   fjne    a
{
        fstsw   ax
        fajne   a
}
macro   fnjne   a
{
        fnstsw  ax
        fajne   a
}
macro   fajne   a
{
        sahf
        jne     a
;       test    ah, $40                         ; 0100 0000
;       jz      a
}

fjnz    fix     fjne                            ; Jump if Not Zero
fnjnz   fix     fnjne                           ; Jump if Not Zero
fajnz   fix     fajne                           ; Jump if Not Zero

; ------------------------------------------------------------------------------

; Jump if C0 flag = 1

macro   fjC0    a
{
        fstsw   ax
        fajC0   a
}
macro   fnjC0   a
{
        fnstsw  ax
        fajC0   a
}
macro   fajC0   a
{
        sahf
        jc      a
;       test    ah, $01                         ; 0000 0001
;       jnz     a
}

; ------------------------------------------------------------------------------

; Jump if C0 flag = 0

macro   fjnC0   a
{
        fstsw   ax
        fajnC0  a
}
macro   fnjnC0  a
{
        fnstsw  ax
        fajnC0  a
}
macro   fajnC0  a
{
        sahf
        jnc     a
;       test    ah, $01                         ; 0000 0001
;       jz      a
}

; ------------------------------------------------------------------------------

; Jump if C1 flag = 1

macro   fjC1    a
{
        fstsw   ax
        fajC1   a
}
macro   fnjC1   a
{
        fnstsw  ax
        fajC1   a
}
macro   fajC1   a
{
        test    ah, $02                         ; 0000 0010
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if C1 flag = 0

macro   fjnC1   a
{
        fstsw   ax
        fajnC1  a
}
macro   fnjnC1  a
{
        fnstsw  ax
        fajnC1  a
}
macro   fajnC1  a
{
        test    ah, $02                         ; 0000 0010
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if C2 flag = 1

macro   fjC2    a
{
        fstsw   ax
        fajC2   a
}
macro   fnjC2   a
{
        fnstsw  ax
        fajC2   a
}
macro   fajC2   a
{
        sahf
        jp      a
;       test    ah, $04                         ; 0000 0100
;       jnz     a
}

; ------------------------------------------------------------------------------

; Jump if C2 flag = 0

macro   fjnC2   a
{
        fstsw   ax
        fajnC2  a
}
macro   fnjnC2  a
{
        fnstsw  ax
        fajnC2  a
}
macro   fajnC2  a
{
        sahf
        jnp     a
;       test    ah, $04                         ; 0000 0100
;       jz      a
}

; ------------------------------------------------------------------------------

; Jump if C3 flag = 1

macro   fjC3    a
{
        fstsw   ax
        fajC3   a
}
macro   fnjC3   a
{
        fnstsw  ax
        fajC3   a
}
macro   fajC3   a
{
        sahf
        jz      a
;       test    ah, $40                         ; 0100 0000
;       jnz     a
}

; ------------------------------------------------------------------------------

; Jump if C3 flag = 0

macro   fjnC3   a
{
        fstsw   ax
        fajnC3  a
}
macro   fnjnC3  a
{
        fnstsw  ax
        fajnC3  a
}
macro   fajnC3  a
{
        sahf
        jnz     a
;       test    ah, $40                         ; 0100 0000
;       jz      a
}

; ------------------------------------------------------------------------------

; Jump if IE flag = 1

macro   fjIE    a
{
        fstsw   ax
        fajIE   a
}
macro   fnjIE   a
{
        fnstsw  ax
        fajIE   a
}
macro   fajIE   a
{
        test    al, $01                         ; 0000 0001
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if IE flag = 0

macro   fjnIE   a
{
        fstsw   ax
        fajnIE  a
}
macro   fnjnIE  a
{
        fnstsw  ax
        fajnIE  a
}
macro   fajnIE  a
{
        test    al, $01                         ; 0000 0001
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if DE flag = 1

macro   fjDE    a
{
        fstsw   ax
        fajDE   a
}
macro   fnjDE   a
{
        fnstsw  ax
        fajDE   a
}
macro   fajDE   a
{
        test    al, $02                         ; 0000 0010
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if DE flag = 0

macro   fjnDE   a
{
        fstsw   ax
        fajnDE  a
}
macro   fnjnDE  a
{
        fnstsw  ax
        fajnDE  a
}
macro   fajnDE  a
{
        test    al, $02                         ; 0000 0010
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if ZE flag = 1

macro   fjZE    a
{
        fstsw   ax
        fajZE   a
}
macro   fnjZE   a
{
        fnstsw  ax
        fajZE   a
}
macro   fajZE   a
{
        test    al, $04                         ; 0000 0100
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if ZE flag = 0

macro   fjnZE   a
{
        fstsw   ax
        fajnZE  a
}
macro   fnjnZE  a
{
        fnstsw  ax
        fajnZE  a
}
macro   fajnZE  a
{
        test    al, $04                         ; 0000 0100
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if OE flag = 1

macro   fjOE    a
{
        fstsw   ax
        fajOE   a
}
macro   fnjOE   a
{
        fnstsw  ax
        fajOE   a
}
macro   fajOE   a
{
        test    al, $08                         ; 0000 1000
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if OE flag = 0

macro   fjnOE   a
{
        fstsw   ax
        fajnOE  a
}
macro   fnjnOE  a
{
        fnstsw  ax
        fajnOE  a
}
macro   fajnOE  a
{
        test    al, $08                         ; 0000 1000
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if UE flag = 1

macro   fjUE    a
{
        fstsw   ax
        fajUE   a
}
macro   fnjUE   a
{
        fnstsw  ax
        fajUE   a
}
macro   fajUE   a
{
        test    al, $10                         ; 0001 0000
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if UE flag = 0

macro   fjnUE   a
{
        fstsw   ax
        fajnUE  a
}
macro   fnjnUE  a
{
        fnstsw  ax
        fajnUE  a
}
macro   fajnUE  a
{
        test    al, $10                         ; 0001 0000
        jz      a
}

; ------------------------------------------------------------------------------

; Jump if PE flag = 1

macro   fjPE    a
{
        fstsw   ax
        fajPE   a
}
macro   fnjPE   a
{
        fnstsw  ax
        fajPE   a
}
macro   fajPE   a
{
        test    al, $20                         ; 0010 0000
        jnz     a
}

; ------------------------------------------------------------------------------

; Jump if PE flag = 0

macro   fjnPE   a
{
        fstsw   ax
        fajnPE  a
}
macro   fnjnPE  a
{
        fnstsw  ax
        fajnPE  a
}
macro   fajnPE  a
{
        test    al, $20                         ; 0010 0000
        jz      a
}

; ------------------------------------------------------------------------------
