
; FASM SOURCE FILE
; http://flatassembler.net

; Floating point routines

; written by @L.CHEMIST (Andrey A. Meshkov)
; maalchemist@gmail.com
; maa-mail@yandex.ru

FLOAT:

; ------------------------------------------------------------------------------

; Returns the integer part of a float number
;
; -> st0 - float argument
; <- st0 - float result
;
; int( 2.1) =  2
; int( 2.5) =  2
; int( 3.5) =  3
; int(-2.1) = -2
; int(-2.5) = -2
; int(-3.5) = -3
$sproc st0int

        add     esp, -4
        fnstcw  [esp]                           ; save control word
        mov     [esp + 2], word 0x0E7F          ; 00001110 01111111
                                                ; trunc toward zero
                                                ; 53-bit precision
                                                ; mask all exceptions
        fldcw   [esp + 2]                       ; load control word
        frndint

    .return:
        fldcw   [esp]                           ; restore control word
        add     esp, 4
        retn
$endp

; ------------------------------------------------------------------------------

; Returns the fractional part of a float number
;
; -> st0 - float argument
; <- st0 - float result
;
; frac( 2.1) =  0.1
; frac( 2.5) =  0.5
; frac( 3.5) =  0.5
; frac(-2.1) = -0.1
; frac(-2.5) = -0.5
; frac(-3.5) = -0.5
$sproc st0frac

        add     esp, -4
        fnstcw  [esp]                           ; save control word
        mov     [esp + 2], word 0x0E7F          ; 00001110 01111111
                                                ; trunc toward zero
                                                ; 53-bit precision
                                                ; mask all exceptions
        fldcw   [esp + 2]                       ; load control word
        fld     st0
        frndint
        fsubp

    .return:
        fldcw   [esp]                           ; restore control word
        add     esp, 4
        retn
$endp

; ------------------------------------------------------------------------------

; Returns the value of X rounded to the nearest whole number.
; Standard round method
; Result = int(X) + int(frac(X) * 2)
;
; -> st0 - float argument (X)
; <- st0 - float result
;
; round( 2.1) =  2
; round( 2.5) =  3
; round( 3.5) =  4
; round(-2.1) = -2
; round(-2.5) = -3
; round(-3.5) = -4
$sproc st0mrnd

        add     esp, -4
        fnstcw  [esp]                           ; save control word
        mov     [esp + 2], word 0x0E7F          ; 00001110 01111111
                                                ; trunc toward zero
                                                ; 53-bit precision
                                                ; mask all exceptions
        fldcw   [esp + 2]                       ; load control word
                                                ; -------------------------------------
                                                ; ST(0)        | ST(1)        | ST(2) |
                                                ; -------------------------------------
                                                ; x.y          | ?            | ?     |
        fld     st0                             ; x.y          | x.y          | ?     |
        fld     st0                             ; x.y          | x.y          | x.y   |
        frndint                                 ; x            | x.y          | x.y   |
        fsubp                                   ; 0.y          | x.y          | ?     |
        fld     st0                             ; 0.y          | 0.y          | x.y   |
        faddp                                   ; 2 * 0.y      | x.y          | ?     |
        frndint                                 ; int(2 * 0.y) | x.y          | ?     |
        fxch                                    ; x.y          | int(2 * 0.y) | ?     |
        frndint                                 ; int(x.y)     | int(2 * 0.y) | ?     |
        faddp                                   ; round(x.y)   | ?            | ?     |
                                                ; -------------------------------------
    .return:
        fldcw   [esp]                           ; restore control word
        add     esp, 4
        retn
$endp

; ------------------------------------------------------------------------------

; Returns the value of X rounded to the nearest whole number.
; If X is exactly halfway between two whole numbers, the result is always the even number.
; This method of rounding is often called "Banker's Rounding".
;
; -> st0 - float argument (X)
; <- st0 - float result

$sproc st0brnd

        add     esp, -4
        fnstcw  [esp]                           ; save control word
        mov     [esp + 2], word 0x027F          ; 00000010 01111111
                                                ; trunc toward nearest
                                                ; 53-bit precision
                                                ; mask all exceptions
        fldcw   [esp + 2]                       ; load control word
        frndint

    .return:
        fldcw   [esp]                           ; restore control word
        add     esp, 4
        retn
$endp

; ------------------------------------------------------------------------------

; Returns the value of X rounded to the nearest whole number
;
; -> st0     - float argument (X)
; <- edx:eax - 64-bit integer result

$sproc st0irnd

        add     esp, -8
        fistp   qword [esp]
        pop     eax
        pop     edx
        fnclex                                  ; clear all exceptions
        retn
$endp

; ------------------------------------------------------------------------------

;  -> st0 - float argument / 11.123 / 11.1235 / 1.9999
;  <- eax -                / 11     / 11      / 2
;  <- edx -                / 123    / 124     / 0

$sproc st0split3d

        add     esp, -8
        mov     [esp + 4], dword 1000
        fld     st0
        call    st0int
        fist    dword [esp]
        fnstsw  ax
        and     ax, 1
        test    ax, ax
        je      @F
        xor     eax, eax
        xor     edx, edx
        jmp    .return
    @@: 
        mov     eax, [esp]
        fsubp
        fabs
        fimul   dword [esp + 4]
        call    st0mrnd
        fistp   dword [esp]
        mov     edx, [esp]
        cmp     edx, [esp + 4]
        jl      @F
        inc     eax
        sub     edx, [esp + 4]
    @@:

    .return:
        add     esp, 8
        fnclex                                  ; clear all exceptions
        retn
$endp

; ------------------------------------------------------------------------------
